# multikit "invoked command" for SDP-specified "audio" programs
# It is a stub for an helper program that has a different argument signature
# (otherwise we could have just invoked the helper program directly)
# version: 1999.05.15

# The main routine (invoked command):

proc mk_sdp_audio {ourProgId ourInternetAddress ourPortNum ourTTL ourPublicKey args} {
	array set attributeArgs $args
	set medium audio

	set sessionName "SDP $medium session"
	catch {set sessionName $attributeArgs(nickname)}

	# protocol specification
	set protocol rtp
	catch {set protocol [string tolower $attributeArgs(sdp_transport)]}
	if {$protocol == "rtp" || $protocol == "rtp/avp"} {
		set rtpFlag "-r"
	} else {
		set rtpFlag ""
	}

	# format specification
	set formats ""
	catch {set formats $attributeArgs(sdp_formats)}
	# I'm not sure how we're supposed to handle more than one format:
	if {[llength $formats] > 1} {
		puts "mk_sdp_$medium warning: multiple formats \"$formats\" for session \"$sessionName\""
	}
	set format [lindex $formats 0]
	
	# misc attributes
	set sdp_attributes ""
	catch {set sdp_attributes $attributeArgs(sdp_media_attributes)}
        set mode ""
        foreach a $sdp_attributes {
	        switch -glob $a {
                        lecture		{set mode "-l"}
			rtpred1:*	{
				# Hack for "rat"; use this as the format
				set format [string range $a \
					     [string length "rtpred1:"] \
					     end]
				set format [string trim $format]
				}
                }
        }

	# convert known numeric formats to string names used by "rat"/"vat":
	switch -exact $format {
		0	{set format pcm}
		3	{set format gsm}
		5	{set format dvi}
		7	{set format lpc}
		14	{ ;# MPEG audio.  "rat"/"vic" doesn't yet handle this, so
			global tcl_platform
if 0 { ;# direct RTP support in "freeamp" no longer works
			if {[string compare $tcl_platform(platform) "windows"] == 0} {
				set cmdName {c:\program files\freeamp.org\freeamp 1.2\freeamp}
			} else {
				set cmdName freeamp
			}
			  if {![catch {exec $cmdName rtp://$ourInternetAddress:$ourPortNum &} pid]} {
				  noteSpawnedProcess $ourInternetAddress $ourPortNum $ourTTL $pid
				  return
			  }
}

			  # NOTE: If you want to run some other command to
			  # read from "playRTPMPEG"s built-in HTTP server,
			  # then change the following.  (Note that the
			  # quotes and braces are important.)
			  if {[string compare $tcl_platform(platform) "windows"] == 0} {
			      set postCmd {"{c:\\program files\\winamp\\winamp} http://localhost:10006"}
			  } else {
			      set postCmd {"{freeamp} http://localhost:10006"}
			  }

			  set execCmd "playRTPMPEG -h 10006 $postCmd -f $format -C [list $sessionName] $ourInternetAddress/$ourPortNum $ourTTL"
			  if {![catch {eval exec $execCmd &} pid]} {
			      noteSpawnedProcess $ourInternetAddress $ourPortNum $ourTTL $pid
			      return
			  }
			}
	}
	# ignore other numeric formats (probably not the right thing to do...)
	if {![catch {expr {int($format)}}]} {
		set format ""
	}

	if {$format != ""} {
		set formatFlag "-f $format"
	} else {
		set formatFlag ""
	}

	# If the protocol is "xing", then try running "swplayer" first:
	if {$protocol == "xing"} {
		set flag "xdma://$ourInternetAddress:$ourPortNum/"

		# Note that the Windows version of "swplayer" takes a filename
		# argument, so we need to create a temp file for it to use:
		global tcl_platform
		if {$tcl_platform(platform) == "windows"} {
			set tmpFileName ~/swplayer.tmp
			set tmpFid [open $tmpFileName "w"]
			puts $tmpFid "stream=$flag"
			close $tmpFid
			set flag [glob $tmpFileName]
		}
		set execCmd "swplayer $flag"
		if {![catch {eval exec $execCmd &} pid]} {
			noteSpawnedProcess $ourInternetAddress $ourPortNum \
				$ourTTL $pid
			return
		}
	}

	# First try running the "rat" tool (unless the protocol is "vat")
	# If this fails, try running "vat" instead
	if {$protocol != "vat"} {
		set execCmd "rat $mode $formatFlag -t $ourTTL -C [list $sessionName] $ourInternetAddress/$ourPortNum"
		if {![catch {eval exec $execCmd &} pid]} {
			noteSpawnedProcess $ourInternetAddress $ourPortNum \
				$ourTTL $pid
			return
		}
	}

	set execCmd "vat $mode $rtpFlag $formatFlag -t $ourTTL -C [list $sessionName] $ourInternetAddress/$ourPortNum"
	if {![catch {eval exec $execCmd &} pid]} {
		noteSpawnedProcess $ourInternetAddress $ourPortNum $ourTTL $pid
		return
	}

	tk_messageBox -title "mk_sdp_$medium: error" \
		-message "Failed to exec \"$execCmd\": $pid" \
		-icon warning -type ok
}

proc noteSpawnedProcess {addr port ttl pid} {
	# Now that we've successfully started a background process to implement
	# the invoked command, we inform multikit of the group/port(s) that
	# we're using (in case they need to be tunneled):
      set groupsocks [multikit_createRTPgroupsocks $addr $port $ttl]
	foreach gs $groupsocks {
		groupsock_turnOnBackgroundReadHandling $gs
	}

	# Finally, periodically check for the death of the spawned process:
	checkSpawnedProcess $pid $groupsocks
}

proc checkSpawnedProcess {pid groupsocks} {
	if {![processExists $pid]} {
		eval multikit_deleteRTPgroupsocks $groupsocks
	} else {
		after 5000 [list checkSpawnedProcess $pid $groupsocks]
	}
}
